# -*- coding: utf-8 -*-
import shutil
from quixote import cleanup

from wcs.ctl.hobo_notify import CmdHoboNotify
from wcs.roles import Role

from utilities import create_temporary_pub

pub = None


def setup_module(module):
    cleanup()
    global pub
    pub = create_temporary_pub()
    pub.cfg['sp'] = {'saml2_providerid': 'test'}
    pub.write_cfg()


def teardown_module(module):
    shutil.rmtree(pub.APP_DIR)


def setup_function(function):
    r = Role(name='Service étt civil')
    r.slug = 'service-ett-civil'
    r.store()


def teardown_function(function):
    Role.wipe()


def test_process_notification_wrong_audience():
    notification = {
        '@type': u'provision',
        'audience': [u'coin'],
        'full': True,
        'objects': [
            {
                '@type': 'role',
                'name': u'Service enfance',
                'slug': u'service-enfance',
                'description': u'Rôle du service petite enfance',
                'uuid': u'12345',
                'emails': [u'petite-enfance@example.com'],
                'emails_to_members': False,
            },
            {
                '@type': 'role',
                'name': u'Service état civil',
                'slug': u'service-etat-civil',
                'description': u'Rôle du service état civil',
                'uuid': u'xyz',
                'emails': [u'etat-civil@example.com'],
                'emails_to_members': True,
            },
        ]
    }
    assert Role.count() == 1
    assert Role.select()[0].name == 'Service étt civil'
    assert Role.select()[0].slug == 'service-ett-civil'
    assert Role.select()[0].details is None
    assert Role.select()[0].emails is None
    assert Role.select()[0].emails_to_members is False
    CmdHoboNotify.process_notification(notification)
    assert Role.count() == 1
    assert Role.select()[0].name == 'Service étt civil'
    assert Role.select()[0].slug == 'service-ett-civil'
    assert Role.select()[0].details is None
    assert Role.select()[0].emails is None
    assert Role.select()[0].emails_to_members is False


def test_process_notification():
    notification = {
        '@type': u'provision',
        'audience': [u'test'],
        'full': True,
        'objects': [
            {
                '@type': 'role',
                'name': u'Service enfance',
                'slug': u'service-enfance',
                'description': u'Rôle du service petite enfance',
                'uuid': u'12345',
                'emails': [u'petite-enfance@example.com'],
                'emails_to_members': False,
            },
            {
                '@type': 'role',
                'name': u'Service état civil',
                'slug': u'service-ett-civil',
                'description': u'Rôle du service état civil',
                'uuid': u'xyz',
                'emails': [u'etat-civil@example.com'],
                'emails_to_members': True,
            },
        ]
    }
    assert Role.count() == 1
    assert Role.select()[0].name == 'Service étt civil'
    assert Role.select()[0].slug == 'service-ett-civil'
    assert Role.select()[0].details is None
    assert Role.select()[0].emails is None
    assert Role.select()[0].emails_to_members is False
    existing_role_id = Role.select()[0].id
    CmdHoboNotify.process_notification(notification)
    assert Role.count() == 2
    old_role = Role.get(existing_role_id)
    assert old_role.name == 'Service état civil'
    assert old_role.slug == 'xyz'
    assert old_role.details == 'Rôle du service état civil'
    assert old_role.emails == ['etat-civil@example.com']
    assert old_role.emails_to_members is True
    new_role = Role.get('12345')
    assert new_role.name == 'Service enfance'
    assert new_role.slug == '12345'
    assert new_role.details == 'Rôle du service petite enfance'
    assert new_role.emails == ['petite-enfance@example.com']
    assert new_role.emails_to_members is False
    notification = {
        '@type': u'provision',
        'audience': [u'test'],
        'full': True,
        'objects': [
            {
                '@type': 'role',
                'name': u'Service enfance',
                'slug': u'service-enfance',
                'description': u'Rôle du service petite enfance',
                'uuid': u'12345',
                'emails': [u'petite-enfance@example.com'],
                'emails_to_members': True,
            },
        ]
    }
    CmdHoboNotify.process_notification(notification)
    assert Role.count() == 1
    assert Role.select()[0].id == new_role.id
    assert Role.select()[0].name == 'Service enfance'
    assert Role.select()[0].slug == '12345'
    assert Role.select()[0].details == 'Rôle du service petite enfance'
    assert Role.select()[0].emails == ['petite-enfance@example.com']
    assert Role.select()[0].emails_to_members is True

def test_process_notification_deprovision():
    notification = {
        '@type': u'deprovision',
        'audience': [u'test'],
        'full': True,
        'objects': [
            {
                '@type': 'role',
                'name': u'Service état civil',
                'slug': u'service-ett-civil',
                'description': u'Rôle du service état civil',
                'uuid': u'xyz',
                'emails': [u'etat-civil@example.com'],
                'emails_to_members': True,
            },
        ]
    }
    assert Role.count() == 1
    assert Role.select()[0].name == 'Service étt civil'
    assert Role.select()[0].slug == 'service-ett-civil'
    CmdHoboNotify.process_notification(notification)
    assert Role.count() == 0
